
//		SetFocusCmd.cpp  --  implements the SETFOCUS command


#ifndef SETFOCUS_CMD
#define SETFOCUS_CMD				1.0.0.1


#include "FindMyWindow.cpp"
#include "ForceFGWin.cpp"



const wchar_t SetFocusHelpText[] = L"Give focus to the specified window.\n\
\n\
SETFOCUS /C:class /Q \"title\"\r\n\n\
\t/C:  the window class name\n\
\t/Q   quietly\n\n\
The title and class names may contain wildcards.  If no title or\n\
class is specified, TCC will be given the focus.\n";


const wchar_t SetFocus_WinTypes[6][17] =	{ L"unknown", L"console", L"Take Command", L"Console2", L"ConEmu", L"Windows Terminal" };



LPCTSTR SetFocus_WinTypeStr( unsigned int Type )
{
	if ( Type <= MY_WINDOW_CONEMU )
		return SetFocus_WinTypes[Type];
	else if ( Type == MY_WINDOW_WINDOWS_TERMINAL )
		return SetFocus_WinTypes[5];
	else
		return SetFocus_WinTypes[0];
}



DLLExports int WINAPI setfocus( LPTSTR inString )
{
	if ( !inString )
		return -1;


	wchar_t DefaultWindowName[] = L"*";

	LPTSTR WindowName			= NULL;
	LPTSTR ClassName			= NULL;
	BOOL Quiet					= FALSE;
	HWND WindowHandle			= NULL;
	unsigned int Type			= 0;


	ParseArgs( inString, PARSE_BREAK_SPACES | PARSE_ONE_ARG_KLUDGE );


	for ( int i = 0; i < numargs; i++ ) {
		if ( arglen[i] ) {
			if ( !wcsncmp( arg[i], L"/?", 2 ) )
				return ShowCmdHelp( SetFocusHelpText, ( arg[i][2] == '?' ), __FILE__, MACRO_STR( SETFOCUS_CMD ) );
			else if ( !_wcsnicmp( arg[i], L"/C", 2 ) ) {
				int j = 2;
				if ( IsColonOrEquals( arg[i][j] ) )
					j++;

				if ( arg[i][j] )
					ClassName = arg[i] + j;
				else
					ClassName = NULL;
			}
			else if ( !_wcsicmp( arg[i], L"/Q" ) )
				Quiet = TRUE;
			else if ( arg[i][0] == '/' ) {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Unknown option: \"%s\"\r\n", arg[i] );
				return 1;
			}
			else if ( !WindowName )
				WindowName = arg[i];
			else {
				CharUpperW( arg[i] );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Syntax error: \"%s\"\r\n", arg[i] );
				return 1;
			}
		}
	}


	if ( ClassName && !WindowName )
		WindowName = DefaultWindowName;


	if ( WindowName ) {
		if ( !ParseInt( WindowName, (unsigned int*) &WindowHandle, NULL ) )
			if ( !IsWindow( WindowHandle ) )
				WindowHandle = NULL;

		if ( !WindowHandle )
			WindowHandle = FuzzyFindWindow( WindowName, ClassName );

		if ( !IsWindow( WindowHandle ) ) {
			DisplayErrorHeader();

			if ( ClassName )
				Qprintf( ERRHANDLE, L"Can\u2019t find window \"%s\" of class \"%s\"\r\n", WindowName, ClassName );
			else
				Qprintf( ERRHANDLE, L"Can\u2019t find window \"%s\"\r\n", WindowName );

			return 2;
		}

		if ( !Quiet )
			Printf( L"Setting focus to window 0x%08X.\r\n", WindowHandle );
	}
	else {
		WindowHandle = FindMyWindow( &Type, NULL );

		if ( !WindowHandle ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Can\u2019t find my window\r\n" );
			return 2;
		}

		if ( !Quiet )
			Printf( L"Setting focus to %s window 0x%08X.\r\n", SetFocus_WinTypeStr( Type ), WindowHandle );
	}

	ForceForegroundWindow( WindowHandle );
	return 0;
}


#endif	//	SETFOCUS_CMD


/*

	1.0.0.0		2024-01-04
				Rewritten to use FindMyWindow.cpp.

	1.0.0.1		2024-03-13
				#include "ForceFGWin.cpp"

*/
